import ddf.minim.spi.*;
import ddf.minim.signals.*;
import ddf.minim.*;
import ddf.minim.analysis.*;
import ddf.minim.ugens.*;
import ddf.minim.effects.*;

Minim minim;
AudioPlayer sound;
//AudioInput sound;
FFT fft; //Fast-Fourier Transform (frequency analysis)

int bufferSize = 512;

void setup()
{
  size(bufferSize, 300);
  minim = new Minim(this);
  
  //* Sound File
  sound = minim.loadFile("test1.mp3", bufferSize);
  //sound.play();
  sound.loop();
  //*/
  
  /* Sound Input
  sound = minim.getLineIn(Minim.STEREO, bufferSize);
  sound.enableMonitoring();
  //*/
  
  fft = new FFT(bufferSize, sound.sampleRate());
}

void draw()
{  
  background(255);
  
  int offset = 100;
  int scale = 50;

  float leftAvg = 0, rightAvg = 0;
  float leftMax = 0, rightMax = 0;
  for(int i = 0; i < sound.bufferSize(); i++) {
    float leftSampleMag = abs(sound.left.get(i));
    leftAvg += leftSampleMag;
    if(leftSampleMag > leftMax) leftMax = leftSampleMag;
    
    float rightSampleMag = abs(sound.right.get(i));
    rightAvg += rightSampleMag;
    if(rightSampleMag > rightMax) rightMax = rightSampleMag;
  }
  leftAvg /= sound.bufferSize();
  rightAvg /= sound.bufferSize();
  
  noStroke();
  int size = 400;
  fill(255, 50, 50, 255*leftMax);
  ellipse(width/4, height/2, size*leftAvg, size*leftAvg);
  //ellipse(width/4, height/2, size*leftMax, size*leftMax);
  fill(20, 100, 255, 255*rightMax);
  ellipse(3*width/4, height/2, size*rightAvg, size*rightAvg);
  //ellipse(3*width/4, height/2, size*rightMax, size*rightMax);
  
  stroke(0);
  noFill();
  fft.forward(sound.mix); //analyse content of the left/right mix
  int maxBand = 0;
  float maxAmp = 0;
  for(int i = 0; i < fft.specSize(); i++){
    float amp = fft.getBand(i);
     if(amp > maxAmp){
       maxAmp = amp;
       maxBand = i;
     }
     int xPos = floor(map(i, 0, fft.specSize(), 0, width));
     line(xPos, height, xPos, height - amp*10);
  }
  
  int w = floor(map(maxBand, 0, fft.specSize()/20, 0, width/2));
  int h = floor(map(maxAmp, 0, fft.specSize(), 5,50));
  int f = floor(map(maxAmp, 0, fft.specSize(), 128,255));
  noStroke();
  fill(f);
  rect(width/2-w/2, height-70, w, h);
  
}

void stop() // required when using Minium
{
  sound.close();
  minim.stop();
  super.stop();
}


